@@ -146,6 +146,7 @@ local_pending_info_free (LocalPendingInfo *info)
 
 struct _TpGroupMixinClassPrivate {
     TpGroupMixinRemMemberWithReasonFunc remove_with_reason;
+    unsigned allow_self_removal : 1;
 };
 
 struct _TpGroupMixinPrivate {
@@ -249,6 +250,39 @@ tp_group_mixin_class_init (GObjectClass *obj_cls,
   mixin_cls->priv = g_slice_new0 (TpGroupMixinClassPrivate);
 }
 
+/**
+ * tp_group_mixin_class_allow_self_removal:
+ * @obj_cls: The class of an object implementing the group interface using this
+ *  mixin
+ *
+ * Configure the mixin to allow attempts to remove the SelfHandle from this
+ * Group, even if the group flags would otherwise disallow this. The
+ * channel's #TpGroupMixinRemMemberFunc or
+ * #TpGroupMixinRemMemberWithReasonFunc will be called as usual for such
+ * attempts, and may make them fail with %TP_ERROR_PERMISSION_DENIED if
+ * required.
+ *
+ * This function should be called from the GObject @class_init callback,
+ * after calling tp_group_mixin_class_init().
+ *
+ * (Recent telepathy-spec changes make it valid to try to remove the
+ * self-handle at all times, regardless of group flags. However, if this was
+ * implemented automatically in TpGroupMixin, this would risk crashing
+ * connection manager implementations that assume that TpGroupMixin will
+ * enforce the group flags strictly. As a result, connection managers should
+ * call this function to indicate to the TpGroupMixin that it may call their
+ * removal callback with the self-handle regardless of flag settings.)
+ *
+ * Since: 0.7.27
+ */
+void
+tp_group_mixin_class_allow_self_removal (GObjectClass *obj_cls)
+{
+  TpGroupMixinClass *mixin_cls = TP_GROUP_MIXIN_CLASS (obj_cls);
+
+  mixin_cls->priv->allow_self_removal = TRUE;
+}
+
 /**
  * tp_group_mixin_init:
  * @obj: An object implementing the group interface using this mixin
@@ -546,6 +580,12 @@ tp_group_mixin_add_members (GObject *obj,
           continue;
         }
 
+      if (mixin_cls->add_member == NULL)
+        {
+          g_set_error (error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
+              "Adding members to this Group channel is not possible");
+          return FALSE;
+        }
       if (!mixin_cls->add_member (obj, handle, message, error))
         {
           return FALSE;
@@ -638,7 +678,16 @@ tp_group_mixin_remove_members_with_reason (GObject *obj,
     {
       handle = g_array_index (contacts, TpHandle, i);
 
-      if (tp_handle_set_is_member (mixin->members, handle))
+      if (mixin_cls->priv->allow_self_removal &&
+          handle == mixin->self_handle &&
+          (tp_handle_set_is_member (mixin->members, handle) ||
+           tp_handle_set_is_member (mixin->remote_pending, handle) ||
+           tp_handle_set_is_member (mixin->local_pending, handle)))
+        {
+          /* don't check the flags - attempting to remove the self-handle
+           * is explicitly always allowed by this channel */
+        }
+      else if (tp_handle_set_is_member (mixin->members, handle))
         {
           if ((mixin->group_flags & TP_CHANNEL_GROUP_FLAG_CAN_REMOVE) == 0)
             {
@@ -683,7 +732,7 @@ tp_group_mixin_remove_members_with_reason (GObject *obj,
     {
       handle = g_array_index (contacts, TpHandle, i);
 
-      if (mixin_cls->priv->remove_with_reason)
+      if (mixin_cls->priv->remove_with_reason != NULL)
         {
           if (!mixin_cls->priv->remove_with_reason (obj, handle, message,
                                                     reason, error))
@@ -691,13 +740,19 @@ tp_group_mixin_remove_members_with_reason (GObject *obj,
               return FALSE;
             }
         }
-      else
+      else if (mixin_cls->remove_member != NULL)
         {
           if (!mixin_cls->remove_member (obj, handle, message, error))
             {
               return FALSE;
             }
         }
+      else
+        {
+          g_set_error (error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
+              "Removing contacts from this Group channel is not possible");
+          return FALSE;
+        }
     }
 
   return TRUE;
@@ -1338,7 +1393,6 @@ maybe_add_member_ids (TpGroupMixin *mixin,
                       TpHandle actor,
                       GHashTable *details)
 {
-  GValue *v;
   GHashTable *member_ids;
 
   /* If the library user had its own ideas about which members' IDs to include
@@ -1364,9 +1418,9 @@ maybe_add_member_ids (TpGroupMixin *mixin,
       g_hash_table_insert (member_ids, GUINT_TO_POINTER (actor), (gchar *) id);
     }
 
-  v = tp_g_value_slice_new (TP_HASH_TYPE_HANDLE_IDENTIFIER_MAP);
-  g_value_take_boxed (v, member_ids);
-  g_hash_table_insert (details, "member-ids", v);
+  g_hash_table_insert (details, "member-ids",
+      tp_g_value_slice_new_take_boxed (TP_HASH_TYPE_HANDLE_IDENTIFIER_MAP,
+          member_ids));
 
   return TRUE;
 }
@@ -1675,28 +1729,24 @@ tp_group_mixin_change_members (GObject *obj,
 {
   GHashTable *details = g_hash_table_new_full (g_str_hash, g_str_equal,
       NULL, (GDestroyNotify) tp_g_value_slice_free);
-  GValue *detail;
   gboolean ret;
 
   if (actor != 0)
     {
-      detail = tp_g_value_slice_new (G_TYPE_UINT);
-      g_value_set_uint (detail, actor);
-      g_hash_table_insert (details, "actor", detail);
+      g_hash_table_insert (details, "actor",
+          tp_g_value_slice_new_uint (actor));
     }
 
   if (reason != TP_CHANNEL_GROUP_CHANGE_REASON_NONE)
     {
-      detail = tp_g_value_slice_new (G_TYPE_UINT);
-      g_value_set_uint (detail, reason);
-      g_hash_table_insert (details, "change-reason", detail);
+      g_hash_table_insert (details, "change-reason",
+          tp_g_value_slice_new_uint (reason));
     }
 
   if (message != NULL && message[0] != '\0')
     {
-      detail = tp_g_value_slice_new (G_TYPE_STRING);
-      g_value_set_string (detail, message);
-      g_hash_table_insert (details, "message", detail);
+      g_hash_table_insert (details, "message",
+          tp_g_value_slice_new_string (message));
     }
 
   ret = change_members (obj, message, add, del, add_local_pending,
